

package com.dremio.exec.expr.fn.impl;

import org.apache.arrow.memory.ArrowBuf;
import com.dremio.exec.expr.SimpleFunction;
import com.dremio.exec.expr.annotations.FunctionTemplate;
import com.dremio.exec.expr.annotations.Output;
import com.dremio.exec.expr.annotations.Param;
import com.dremio.exec.expr.fn.FunctionErrorContext;
import org.apache.arrow.vector.holders.NullableVarCharHolder;
import org.apache.arrow.vector.holders.VarCharHolder;

import javax.inject.Inject;

/**
 * This file is generated with Freemarker using the template exec/java-exec/src/main/codegen/templates/DirectoryExplorers.java
 */
public class DirectoryExplorers {
  static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(DirectoryExplorers.class);



  @FunctionTemplate(name = "maxdir", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = FunctionTemplate.NullHandling.INTERNAL)
  public static class MaxDir implements SimpleFunction {

    @Param VarCharHolder schema;
    @Param VarCharHolder table;
    @Output NullableVarCharHolder out;
    @Inject ArrowBuf buffer;
    @Inject com.dremio.exec.store.PartitionExplorer partitionExplorer;
    @Inject FunctionErrorContext errorContext;

    public void setup() {
    }

    public void eval() {
      out.isSet = 1;
      Iterable<String> subPartitions;
      try {
        subPartitions = partitionExplorer.getSubPartitions(
            com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(schema),
            com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(table),
            new java.util.ArrayList<String>(),
            new java.util.ArrayList<String>());
      } catch (com.dremio.exec.store.PartitionNotFoundException e) {
        throw errorContext.error(e)
          .message(
            String.format("Error in %s function: Table %s does not exist in schema %s ",
                "maxdir",
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(table),
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(schema))
          )
          .build();
      }
      java.util.Iterator<String> partitionIterator = subPartitions.iterator();
      if (!partitionIterator.hasNext()) {
        throw errorContext.error()
          .message(
            String.format("Error in %s function: Table %s in schema %s does not contain sub-partitions.",
                "maxdir",
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(table),
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(schema)
            )
          )
          .build();
      }
      String subPartitionStr = partitionIterator.next();
      String curr;
      // find the maximum directory in the list using a case-sensitive string comparison
      while (partitionIterator.hasNext()){
        curr = partitionIterator.next();
        if (subPartitionStr.compareTo(curr) < 0) {
          subPartitionStr = curr;
        }
      }
      String[] subPartitionParts = subPartitionStr.split("/");
      subPartitionStr = subPartitionParts[subPartitionParts.length - 1];
      byte[] result = subPartitionStr.getBytes(java.nio.charset.StandardCharsets.UTF_8);
      out.buffer = buffer = buffer.reallocIfNeeded(result.length);

      out.buffer.setBytes(0, subPartitionStr.getBytes(), 0, result.length);
      out.start = 0;
      out.end = result.length;
    }
  }


  @FunctionTemplate(name = "imaxdir", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = FunctionTemplate.NullHandling.INTERNAL)
  public static class IMaxDir implements SimpleFunction {

    @Param VarCharHolder schema;
    @Param VarCharHolder table;
    @Output NullableVarCharHolder out;
    @Inject ArrowBuf buffer;
    @Inject com.dremio.exec.store.PartitionExplorer partitionExplorer;
    @Inject FunctionErrorContext errorContext;

    public void setup() {
    }

    public void eval() {
      out.isSet = 1;
      Iterable<String> subPartitions;
      try {
        subPartitions = partitionExplorer.getSubPartitions(
            com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(schema),
            com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(table),
            new java.util.ArrayList<String>(),
            new java.util.ArrayList<String>());
      } catch (com.dremio.exec.store.PartitionNotFoundException e) {
        throw errorContext.error(e)
          .message(
            String.format("Error in %s function: Table %s does not exist in schema %s ",
                "imaxdir",
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(table),
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(schema))
          )
          .build();
      }
      java.util.Iterator<String> partitionIterator = subPartitions.iterator();
      if (!partitionIterator.hasNext()) {
        throw errorContext.error()
          .message(
            String.format("Error in %s function: Table %s in schema %s does not contain sub-partitions.",
                "imaxdir",
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(table),
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(schema)
            )
          )
          .build();
      }
      String subPartitionStr = partitionIterator.next();
      String curr;
      // find the maximum directory in the list using a case-insensitive string comparison
      while (partitionIterator.hasNext()){
        curr = partitionIterator.next();
        if (subPartitionStr.compareToIgnoreCase(curr) < 0) {
          subPartitionStr = curr;
        }
      }
      String[] subPartitionParts = subPartitionStr.split("/");
      subPartitionStr = subPartitionParts[subPartitionParts.length - 1];
      byte[] result = subPartitionStr.getBytes(java.nio.charset.StandardCharsets.UTF_8);
      out.buffer = buffer = buffer.reallocIfNeeded(result.length);

      out.buffer.setBytes(0, subPartitionStr.getBytes(), 0, result.length);
      out.start = 0;
      out.end = result.length;
    }
  }


  @FunctionTemplate(name = "mindir", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = FunctionTemplate.NullHandling.INTERNAL)
  public static class MinDir implements SimpleFunction {

    @Param VarCharHolder schema;
    @Param VarCharHolder table;
    @Output NullableVarCharHolder out;
    @Inject ArrowBuf buffer;
    @Inject com.dremio.exec.store.PartitionExplorer partitionExplorer;
    @Inject FunctionErrorContext errorContext;

    public void setup() {
    }

    public void eval() {
      out.isSet = 1;
      Iterable<String> subPartitions;
      try {
        subPartitions = partitionExplorer.getSubPartitions(
            com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(schema),
            com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(table),
            new java.util.ArrayList<String>(),
            new java.util.ArrayList<String>());
      } catch (com.dremio.exec.store.PartitionNotFoundException e) {
        throw errorContext.error(e)
          .message(
            String.format("Error in %s function: Table %s does not exist in schema %s ",
                "mindir",
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(table),
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(schema))
          )
          .build();
      }
      java.util.Iterator<String> partitionIterator = subPartitions.iterator();
      if (!partitionIterator.hasNext()) {
        throw errorContext.error()
          .message(
            String.format("Error in %s function: Table %s in schema %s does not contain sub-partitions.",
                "mindir",
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(table),
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(schema)
            )
          )
          .build();
      }
      String subPartitionStr = partitionIterator.next();
      String curr;
      // find the minimum directory in the list using a case-sensitive string comparison
      while (partitionIterator.hasNext()){
        curr = partitionIterator.next();
        if (subPartitionStr.compareTo(curr) > 0) {
          subPartitionStr = curr;
        }
      }
      String[] subPartitionParts = subPartitionStr.split("/");
      subPartitionStr = subPartitionParts[subPartitionParts.length - 1];
      byte[] result = subPartitionStr.getBytes(java.nio.charset.StandardCharsets.UTF_8);
      out.buffer = buffer = buffer.reallocIfNeeded(result.length);

      out.buffer.setBytes(0, subPartitionStr.getBytes(), 0, result.length);
      out.start = 0;
      out.end = result.length;
    }
  }


  @FunctionTemplate(name = "imindir", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = FunctionTemplate.NullHandling.INTERNAL)
  public static class IMinDir implements SimpleFunction {

    @Param VarCharHolder schema;
    @Param VarCharHolder table;
    @Output NullableVarCharHolder out;
    @Inject ArrowBuf buffer;
    @Inject com.dremio.exec.store.PartitionExplorer partitionExplorer;
    @Inject FunctionErrorContext errorContext;

    public void setup() {
    }

    public void eval() {
      out.isSet = 1;
      Iterable<String> subPartitions;
      try {
        subPartitions = partitionExplorer.getSubPartitions(
            com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(schema),
            com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(table),
            new java.util.ArrayList<String>(),
            new java.util.ArrayList<String>());
      } catch (com.dremio.exec.store.PartitionNotFoundException e) {
        throw errorContext.error(e)
          .message(
            String.format("Error in %s function: Table %s does not exist in schema %s ",
                "imindir",
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(table),
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(schema))
          )
          .build();
      }
      java.util.Iterator<String> partitionIterator = subPartitions.iterator();
      if (!partitionIterator.hasNext()) {
        throw errorContext.error()
          .message(
            String.format("Error in %s function: Table %s in schema %s does not contain sub-partitions.",
                "imindir",
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(table),
                com.dremio.exec.expr.fn.impl.StringFunctionHelpers.getStringFromVarCharHolder(schema)
            )
          )
          .build();
      }
      String subPartitionStr = partitionIterator.next();
      String curr;
      // find the minimum directory in the list using a case-insensitive string comparison
      while (partitionIterator.hasNext()){
        curr = partitionIterator.next();
        if (subPartitionStr.compareToIgnoreCase(curr) > 0) {
          subPartitionStr = curr;
        }
      }
      String[] subPartitionParts = subPartitionStr.split("/");
      subPartitionStr = subPartitionParts[subPartitionParts.length - 1];
      byte[] result = subPartitionStr.getBytes(java.nio.charset.StandardCharsets.UTF_8);
      out.buffer = buffer = buffer.reallocIfNeeded(result.length);

      out.buffer.setBytes(0, subPartitionStr.getBytes(), 0, result.length);
      out.start = 0;
      out.end = result.length;
    }
  }
}
