

package com.dremio.exec.expr.fn.impl;

import com.dremio.exec.expr.SimpleFunction;
import com.dremio.exec.expr.annotations.FunctionTemplate;
import com.dremio.exec.expr.annotations.FunctionTemplate.NullHandling;
import com.dremio.exec.expr.annotations.Output;
import com.dremio.exec.expr.annotations.Workspace;
import com.dremio.exec.expr.annotations.Param;
import org.apache.arrow.vector.holders.*;

import io.netty.buffer.ByteBuf;

/**
 * generated from DateIntervalFunctionTemplates/DateTruncFunctions.java
 */
public class GDateTimeTruncateFunctions {

 

  /**
   * This class merely act as a placeholder so that Calcite allows 'trunc('truncationUnit', col)'
   * function in SQL.
   */
  @SuppressWarnings("unused")
  @FunctionTemplate(name = "date_trunc", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GDateMilliDummyDateTrunc implements SimpleFunction {
    @Param  VarCharHolder left;
    @Param  DateMilliHolder right;
    @Output DateMilliHolder out;

    @Override
    public void setup() { }

    @Override
    public void eval() { }
  }
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Second", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GDateMilliSecondDateTrunc implements SimpleFunction {
    @Param  DateMilliHolder right;
    @Output DateMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
               
      // No truncation as there is no time part in date
      out.value = right.value;
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Minute", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GDateMilliMinuteDateTrunc implements SimpleFunction {
    @Param  DateMilliHolder right;
    @Output DateMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
               
      // No truncation as there is no time part in date
      out.value = right.value;
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Hour", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GDateMilliHourDateTrunc implements SimpleFunction {
    @Param  DateMilliHolder right;
    @Output DateMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
               
      // No truncation as there is no time part in date
      out.value = right.value;
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Day", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GDateMilliDayDateTrunc implements SimpleFunction {
    @Param  DateMilliHolder right;
    @Output DateMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
               
      // No truncation as there is no time part in date
      out.value = right.value;
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Month", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GDateMilliMonthDateTrunc implements SimpleFunction {
    @Param  DateMilliHolder right;
    @Output DateMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
      dateTime.setMillis(right.value);
      dateTime.setRounding(dateTime.getChronology().monthOfYear());
      out.value = dateTime.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Year", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GDateMilliYearDateTrunc implements SimpleFunction {
    @Param  DateMilliHolder right;
    @Output DateMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
      dateTime.setMillis(right.value);
      dateTime.setRounding(dateTime.getChronology().year());
      out.value = dateTime.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Week", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GDateMilliWeekDateTrunc implements SimpleFunction {
    @Param  DateMilliHolder right;
    @Output DateMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
      dateTime.setMillis(right.value);
      dateTime.setRounding(dateTime.getChronology().weekOfWeekyear());
      out.value = dateTime.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Quarter", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GDateMilliQuarterDateTrunc implements SimpleFunction {
    @Param  DateMilliHolder right;
    @Output DateMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
      dateTime.setMillis(right.value);
      int month = dateTime.getMonthOfYear();
      dateTime.setRounding(dateTime.getChronology().year());
      dateTime2.setMillis(dateTime.getMillis());
      dateTime2.add(org.joda.time.DurationFieldType.months(), ((month-1)/3)*3);
      out.value = dateTime2.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Decade", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GDateMilliDecadeDateTrunc implements SimpleFunction {
    @Param  DateMilliHolder right;
    @Output DateMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
      dateTime.setMillis(right.value);
      int year = dateTime.getYear();
      dateTime.setRounding(dateTime.getChronology().centuryOfEra());
      dateTime2.setMillis(dateTime.getMillis());
      dateTime2.add(org.joda.time.DurationFieldType.years(), ((year%100)/10)*10);
      out.value = dateTime2.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Century", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GDateMilliCenturyDateTrunc implements SimpleFunction {
    @Param  DateMilliHolder right;
    @Output DateMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
      dateTime.setMillis(right.value);
      dateTime.add(org.joda.time.DurationFieldType.years(), -1);
      dateTime.setRounding(dateTime.getChronology().centuryOfEra());
      dateTime2.setMillis(dateTime.getMillis());
      dateTime2.add(org.joda.time.DurationFieldType.years(), 1);
      out.value = dateTime2.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Millennium", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GDateMilliMillenniumDateTrunc implements SimpleFunction {
    @Param  DateMilliHolder right;
    @Output DateMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
      dateTime.setMillis(right.value);
      int year = dateTime.getYear();
      dateTime.setRounding(dateTime.getChronology().era());
      dateTime2.setMillis(dateTime.getMillis());
      dateTime2.add(org.joda.time.DurationFieldType.years(), ((year-1)/1000)*1000);
      out.value = dateTime2.getMillis();
         
 
    }
}
    
   
 

  /**
   * This class merely act as a placeholder so that Calcite allows 'trunc('truncationUnit', col)'
   * function in SQL.
   */
  @SuppressWarnings("unused")
  @FunctionTemplate(name = "date_trunc", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeStampMilliDummyDateTrunc implements SimpleFunction {
    @Param  VarCharHolder left;
    @Param  TimeStampMilliHolder right;
    @Output TimeStampMilliHolder out;

    @Override
    public void setup() { }

    @Override
    public void eval() { }
  }
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Second", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeStampMilliSecondDateTrunc implements SimpleFunction {
    @Param  TimeStampMilliHolder right;
    @Output TimeStampMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
            dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
              dateTime.setMillis(right.value);
      dateTime.setRounding(dateTime.getChronology().secondOfMinute());
      out.value = dateTime.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Minute", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeStampMilliMinuteDateTrunc implements SimpleFunction {
    @Param  TimeStampMilliHolder right;
    @Output TimeStampMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
            dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
              dateTime.setMillis(right.value);
      dateTime.setRounding(dateTime.getChronology().minuteOfHour());
      out.value = dateTime.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Hour", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeStampMilliHourDateTrunc implements SimpleFunction {
    @Param  TimeStampMilliHolder right;
    @Output TimeStampMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
            dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
              dateTime.setMillis(right.value);
      dateTime.setRounding(dateTime.getChronology().hourOfDay());
      out.value = dateTime.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Day", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeStampMilliDayDateTrunc implements SimpleFunction {
    @Param  TimeStampMilliHolder right;
    @Output TimeStampMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
            dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
              dateTime.setMillis(right.value);
      dateTime.setRounding(dateTime.getChronology().dayOfMonth());
      out.value = dateTime.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Month", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeStampMilliMonthDateTrunc implements SimpleFunction {
    @Param  TimeStampMilliHolder right;
    @Output TimeStampMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
            dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
              dateTime.setMillis(right.value);
      dateTime.setRounding(dateTime.getChronology().monthOfYear());
      out.value = dateTime.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Year", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeStampMilliYearDateTrunc implements SimpleFunction {
    @Param  TimeStampMilliHolder right;
    @Output TimeStampMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
            dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
         
      dateTime.setMillis(right.value);
      dateTime.setRounding(dateTime.getChronology().year());
      out.value = dateTime.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Week", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeStampMilliWeekDateTrunc implements SimpleFunction {
    @Param  TimeStampMilliHolder right;
    @Output TimeStampMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
            dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
              dateTime.setMillis(right.value);
      dateTime.setRounding(dateTime.getChronology().weekOfWeekyear());
      out.value = dateTime.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Quarter", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeStampMilliQuarterDateTrunc implements SimpleFunction {
    @Param  TimeStampMilliHolder right;
    @Output TimeStampMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
            dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
              dateTime.setMillis(right.value);
      int month = dateTime.getMonthOfYear();
      dateTime.setRounding(dateTime.getChronology().year());
      dateTime2.setMillis(dateTime.getMillis());
      dateTime2.add(org.joda.time.DurationFieldType.months(), ((month-1)/3)*3);
      out.value = dateTime2.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Decade", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeStampMilliDecadeDateTrunc implements SimpleFunction {
    @Param  TimeStampMilliHolder right;
    @Output TimeStampMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
            dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
              dateTime.setMillis(right.value);
      int year = dateTime.getYear();
      dateTime.setRounding(dateTime.getChronology().centuryOfEra());
      dateTime2.setMillis(dateTime.getMillis());
      dateTime2.add(org.joda.time.DurationFieldType.years(), ((year%100)/10)*10);
      out.value = dateTime2.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Century", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeStampMilliCenturyDateTrunc implements SimpleFunction {
    @Param  TimeStampMilliHolder right;
    @Output TimeStampMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
            dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
              dateTime.setMillis(right.value);
      dateTime.add(org.joda.time.DurationFieldType.years(), -1);
      dateTime.setRounding(dateTime.getChronology().centuryOfEra());
      dateTime2.setMillis(dateTime.getMillis());
      dateTime2.add(org.joda.time.DurationFieldType.years(), 1);
      out.value = dateTime2.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Millennium", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeStampMilliMillenniumDateTrunc implements SimpleFunction {
    @Param  TimeStampMilliHolder right;
    @Output TimeStampMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
            dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
              dateTime.setMillis(right.value);
      int year = dateTime.getYear();
      dateTime.setRounding(dateTime.getChronology().era());
      dateTime2.setMillis(dateTime.getMillis());
      dateTime2.add(org.joda.time.DurationFieldType.years(), ((year-1)/1000)*1000);
      out.value = dateTime2.getMillis();
         
 
    }
}
    
   
 

  /**
   * This class merely act as a placeholder so that Calcite allows 'trunc('truncationUnit', col)'
   * function in SQL.
   */
  @SuppressWarnings("unused")
  @FunctionTemplate(name = "date_trunc", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeMilliDummyDateTrunc implements SimpleFunction {
    @Param  VarCharHolder left;
    @Param  TimeMilliHolder right;
    @Output TimeMilliHolder out;

    @Override
    public void setup() { }

    @Override
    public void eval() { }
  }
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Second", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeMilliSecondDateTrunc implements SimpleFunction {
    @Param  TimeMilliHolder right;
    @Output TimeMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
       
              dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
      dateTime.setMillis(right.value);
      dateTime.setRounding(dateTime.getChronology().secondOfMinute());
      out.value = (int) dateTime.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Minute", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeMilliMinuteDateTrunc implements SimpleFunction {
    @Param  TimeMilliHolder right;
    @Output TimeMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
       
              dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
      dateTime.setMillis(right.value);
      dateTime.setRounding(dateTime.getChronology().minuteOfHour());
      out.value = (int) dateTime.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Hour", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeMilliHourDateTrunc implements SimpleFunction {
    @Param  TimeMilliHolder right;
    @Output TimeMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
       
         
      dateTime.setRounding(null, org.joda.time.MutableDateTime.ROUND_NONE);
      dateTime.setMillis(right.value);
      dateTime.setRounding(dateTime.getChronology().hourOfDay());
      out.value = (int) dateTime.getMillis();
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Day", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeMilliDayDateTrunc implements SimpleFunction {
    @Param  TimeMilliHolder right;
    @Output TimeMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
       
              out.value = 0;
         
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Month", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeMilliMonthDateTrunc implements SimpleFunction {
    @Param  TimeMilliHolder right;
    @Output TimeMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
       
              out.value = 0;
         
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Year", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeMilliYearDateTrunc implements SimpleFunction {
    @Param  TimeMilliHolder right;
    @Output TimeMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
       
              out.value = 0;
         
 
    }
}
    
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Quarter", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeMilliQuarterDateTrunc implements SimpleFunction {
    @Param  TimeMilliHolder right;
    @Output TimeMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
       
              out.value = 0;
         
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Decade", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeMilliDecadeDateTrunc implements SimpleFunction {
    @Param  TimeMilliHolder right;
    @Output TimeMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
       
              out.value = 0;
         
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Century", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeMilliCenturyDateTrunc implements SimpleFunction {
    @Param  TimeMilliHolder right;
    @Output TimeMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
       
              out.value = 0;
         
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Millennium", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GTimeMilliMillenniumDateTrunc implements SimpleFunction {
    @Param  TimeMilliHolder right;
    @Output TimeMilliHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
       
              out.value = 0;
         
 
    }
}
    
   
 

  /**
   * This class merely act as a placeholder so that Calcite allows 'trunc('truncationUnit', col)'
   * function in SQL.
   */
  @SuppressWarnings("unused")
  @FunctionTemplate(name = "date_trunc", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalDayDummyDateTrunc implements SimpleFunction {
    @Param  VarCharHolder left;
    @Param  IntervalDayHolder right;
    @Output IntervalDayHolder out;

    @Override
    public void setup() { }

    @Override
    public void eval() { }
  }
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Second", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalDaySecondDateTrunc implements SimpleFunction {
    @Param  IntervalDayHolder right;
    @Output IntervalDayHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
               
      out.days = right.days;
      out.milliseconds = (right.milliseconds/(org.apache.arrow.vector.util.DateUtility.secondsToMillis))*
        (org.apache.arrow.vector.util.DateUtility.secondsToMillis);
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Minute", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalDayMinuteDateTrunc implements SimpleFunction {
    @Param  IntervalDayHolder right;
    @Output IntervalDayHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.days = right.days;
      out.milliseconds = (right.milliseconds/(org.apache.arrow.vector.util.DateUtility.minutesToMillis))*
          (org.apache.arrow.vector.util.DateUtility.minutesToMillis);
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Hour", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalDayHourDateTrunc implements SimpleFunction {
    @Param  IntervalDayHolder right;
    @Output IntervalDayHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.days = right.days;
      out.milliseconds =
          (right.milliseconds/(org.apache.arrow.vector.util.DateUtility.hoursToMillis))*
              (org.apache.arrow.vector.util.DateUtility.hoursToMillis);
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Day", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalDayDayDateTrunc implements SimpleFunction {
    @Param  IntervalDayHolder right;
    @Output IntervalDayHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.days = right.days;
      out.milliseconds = 0;
 
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Month", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalDayMonthDateTrunc implements SimpleFunction {
    @Param  IntervalDayHolder right;
    @Output IntervalDayHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.days = 0;
      out.milliseconds = 0;
         
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Year", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalDayYearDateTrunc implements SimpleFunction {
    @Param  IntervalDayHolder right;
    @Output IntervalDayHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.days = 0;
      out.milliseconds = 0;
         
 
    }
}
    
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Quarter", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalDayQuarterDateTrunc implements SimpleFunction {
    @Param  IntervalDayHolder right;
    @Output IntervalDayHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.days = 0;
      out.milliseconds = 0;
         
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Decade", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalDayDecadeDateTrunc implements SimpleFunction {
    @Param  IntervalDayHolder right;
    @Output IntervalDayHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.days = 0;
      out.milliseconds = 0;
         
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Century", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalDayCenturyDateTrunc implements SimpleFunction {
    @Param  IntervalDayHolder right;
    @Output IntervalDayHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.days = 0;
      out.milliseconds = 0;
         
 
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Millennium", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalDayMillenniumDateTrunc implements SimpleFunction {
    @Param  IntervalDayHolder right;
    @Output IntervalDayHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.days = 0;
      out.milliseconds = 0;
         
 
    }
}
    
   
 

  /**
   * This class merely act as a placeholder so that Calcite allows 'trunc('truncationUnit', col)'
   * function in SQL.
   */
  @SuppressWarnings("unused")
  @FunctionTemplate(name = "date_trunc", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalYearDummyDateTrunc implements SimpleFunction {
    @Param  VarCharHolder left;
    @Param  IntervalYearHolder right;
    @Output IntervalYearHolder out;

    @Override
    public void setup() { }

    @Override
    public void eval() { }
  }
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Second", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalYearSecondDateTrunc implements SimpleFunction {
    @Param  IntervalYearHolder right;
    @Output IntervalYearHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
               
      out.value = right.value;
 
       
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Minute", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalYearMinuteDateTrunc implements SimpleFunction {
    @Param  IntervalYearHolder right;
    @Output IntervalYearHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
               
      out.value = right.value;
 
       
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Hour", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalYearHourDateTrunc implements SimpleFunction {
    @Param  IntervalYearHolder right;
    @Output IntervalYearHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
               
      out.value = right.value;
 
       
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Day", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalYearDayDateTrunc implements SimpleFunction {
    @Param  IntervalYearHolder right;
    @Output IntervalYearHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
               
      out.value = right.value;
 
       
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Month", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalYearMonthDateTrunc implements SimpleFunction {
    @Param  IntervalYearHolder right;
    @Output IntervalYearHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.value = right.value;
 
       
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Year", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalYearYearDateTrunc implements SimpleFunction {
    @Param  IntervalYearHolder right;
    @Output IntervalYearHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.value = (right.value / 12) * 12;
 
       
    }
}
    
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Quarter", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalYearQuarterDateTrunc implements SimpleFunction {
    @Param  IntervalYearHolder right;
    @Output IntervalYearHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.value = (right.value / 3) * 3;
 
       
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Decade", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalYearDecadeDateTrunc implements SimpleFunction {
    @Param  IntervalYearHolder right;
    @Output IntervalYearHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.value = (right.value / 120) * 120;
 
       
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Century", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalYearCenturyDateTrunc implements SimpleFunction {
    @Param  IntervalYearHolder right;
    @Output IntervalYearHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.value = (right.value / 1200) * 1200;
 
       
    }
}
    
    

  @SuppressWarnings("unused")
  @FunctionTemplate(names = "date_trunc_Millennium", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntervalYearMillenniumDateTrunc implements SimpleFunction {
    @Param  IntervalYearHolder right;
    @Output IntervalYearHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;
    @Workspace org.joda.time.MutableDateTime dateTime2;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
      dateTime2 = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
                    out.value = (right.value / 12000) * 12000;
         
       
    }
}
    
   
 
}
