

package com.dremio.exec.expr.fn.impl;

import com.dremio.exec.expr.SimpleFunction;
import com.dremio.exec.expr.annotations.*;
import org.apache.arrow.vector.holders.*;
import org.apache.arrow.vector.util.DateUtility;

/**
 * generated from DateIntervalFunctionTemplates/Extract.java
 */
public class GExtract {

 
   
  @FunctionTemplate(name = "extractSecond", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class SecondFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getSecondOfMinute();
      out.value += (dateTime.getMillisOfSecond()) / org.apache.arrow.vector.util.DateUtility.secondsToMillis;
    }
  }
   
  @FunctionTemplate(name = "extractMinute", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class MinuteFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getMinuteOfHour();
    }
  }
   
  @FunctionTemplate(name = "extractHour", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class HourFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getHourOfDay();
    }
  }
   
  @FunctionTemplate(name = "extractDay", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class DayFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getDayOfMonth();
    }
  }
   
  @FunctionTemplate(name = "extractMonth", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class MonthFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getMonthOfYear();
    }
  }
   
  @FunctionTemplate(name = "extractYear", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class YearFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getYear();
    }
  }
   
  @FunctionTemplate(name = "extractCentury", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class CenturyFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = (dateTime.getYear() - 1)/100 + 1;
    }
  }
   
  @FunctionTemplate(name = "extractDecade", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class DecadeFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getYear()/10;
    }
  }
   
  @FunctionTemplate(name = "extractDOW", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class DOWFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getDayOfWeek();
    }
  }
   
  @FunctionTemplate(name = "extractDOY", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class DOYFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getDayOfYear();
    }
  }
   
  @FunctionTemplate(name = "extractEpoch", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class EpochFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getMillis()/1000;
    }
  }
   
  @FunctionTemplate(name = "extractMillennium", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class MillenniumFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = (dateTime.getYear() - 1)/1000 + 1;
    }
  }
   
  @FunctionTemplate(name = "extractQuarter", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class QuarterFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = (dateTime.getMonthOfYear()-1)/3 + 1;
    }
  }
   
  @FunctionTemplate(name = "extractWeek", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class WeekFromDateMilli implements SimpleFunction {

    @Param DateMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getWeekOfWeekyear();
    }
  }
   
 
   
  @FunctionTemplate(name = "extractSecond", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class SecondFromTimeMilli implements SimpleFunction {

    @Param TimeMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getSecondOfMinute();
      out.value += (dateTime.getMillisOfSecond()) / org.apache.arrow.vector.util.DateUtility.secondsToMillis;
    }
  }
   
  @FunctionTemplate(name = "extractMinute", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class MinuteFromTimeMilli implements SimpleFunction {

    @Param TimeMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getMinuteOfHour();
    }
  }
   
  @FunctionTemplate(name = "extractHour", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class HourFromTimeMilli implements SimpleFunction {

    @Param TimeMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getHourOfDay();
    }
  }
   
   
   
   
   
   
   
   
  @FunctionTemplate(name = "extractEpoch", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class EpochFromTimeMilli implements SimpleFunction {

    @Param TimeMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getMillis()/1000;
    }
  }
   
   
   
   
 
   
  @FunctionTemplate(name = "extractSecond", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class SecondFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getSecondOfMinute();
      out.value += (dateTime.getMillisOfSecond()) / org.apache.arrow.vector.util.DateUtility.secondsToMillis;
    }
  }
   
  @FunctionTemplate(name = "extractMinute", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class MinuteFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getMinuteOfHour();
    }
  }
   
  @FunctionTemplate(name = "extractHour", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class HourFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getHourOfDay();
    }
  }
   
  @FunctionTemplate(name = "extractDay", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class DayFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getDayOfMonth();
    }
  }
   
  @FunctionTemplate(name = "extractMonth", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class MonthFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getMonthOfYear();
    }
  }
   
  @FunctionTemplate(name = "extractYear", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class YearFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getYear();
    }
  }
   
  @FunctionTemplate(name = "extractCentury", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class CenturyFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = (dateTime.getYear() - 1)/100 + 1;
    }
  }
   
  @FunctionTemplate(name = "extractDecade", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class DecadeFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getYear()/10;
    }
  }
   
  @FunctionTemplate(name = "extractDOW", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class DOWFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getDayOfWeek();
    }
  }
   
  @FunctionTemplate(name = "extractDOY", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class DOYFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getDayOfYear();
    }
  }
   
  @FunctionTemplate(name = "extractEpoch", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class EpochFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getMillis()/1000;
    }
  }
   
  @FunctionTemplate(name = "extractMillennium", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class MillenniumFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = (dateTime.getYear() - 1)/1000 + 1;
    }
  }
   
  @FunctionTemplate(name = "extractQuarter", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class QuarterFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = (dateTime.getMonthOfYear()-1)/3 + 1;
    }
  }
   
  @FunctionTemplate(name = "extractWeek", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class WeekFromTimeStampMilli implements SimpleFunction {

    @Param TimeStampMilliHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(in.value);

      out.value = dateTime.getWeekOfWeekyear();
    }
  }
   
 
   
  @FunctionTemplate(name = "extractSecond", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class SecondFromIntervalDay implements SimpleFunction {

    @Param IntervalDayHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
      long millis = in.milliseconds % org.apache.arrow.vector.util.DateUtility.minutesToMillis;
      out.value = millis / (org.apache.arrow.vector.util.DateUtility.secondsToMillis);
      }
  }
   
  @FunctionTemplate(name = "extractMinute", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class MinuteFromIntervalDay implements SimpleFunction {

    @Param IntervalDayHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
      int millis = in.milliseconds % (org.apache.arrow.vector.util.DateUtility.hoursToMillis);
      out.value = millis / (org.apache.arrow.vector.util.DateUtility.minutesToMillis);
      }
  }
   
  @FunctionTemplate(name = "extractHour", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class HourFromIntervalDay implements SimpleFunction {

    @Param IntervalDayHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
      out.value = in.milliseconds/(org.apache.arrow.vector.util.DateUtility.hoursToMillis);
      }
  }
   
  @FunctionTemplate(name = "extractDay", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class DayFromIntervalDay implements SimpleFunction {

    @Param IntervalDayHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
      out.value = in.days;
      }
  }
   
  @FunctionTemplate(name = "extractMonth", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class MonthFromIntervalDay implements SimpleFunction {

    @Param IntervalDayHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
      out.value = 0;
      }
  }
   
  @FunctionTemplate(name = "extractYear", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class YearFromIntervalDay implements SimpleFunction {

    @Param IntervalDayHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
      out.value = 0;
      }
  }
   
  @FunctionTemplate(name = "extractCentury", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class CenturyFromIntervalDay implements SimpleFunction {

    @Param IntervalDayHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
      out.value = 0;
      }
  }
   
  @FunctionTemplate(name = "extractDecade", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class DecadeFromIntervalDay implements SimpleFunction {

    @Param IntervalDayHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
      out.value = 0;
      }
  }
   
   
   
  @FunctionTemplate(name = "extractEpoch", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class EpochFromIntervalDay implements SimpleFunction {

    @Param IntervalDayHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
      dateTime.setMillis(0);
      dateTime.add(org.joda.time.DurationFieldType.days(), in.days);
      dateTime.add(org.joda.time.DurationFieldType.millis(), in.milliseconds);
      out.value = dateTime.getMillis()/1000;
      }
  }
   
  @FunctionTemplate(name = "extractMillennium", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class MillenniumFromIntervalDay implements SimpleFunction {

    @Param IntervalDayHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
      out.value = 0;
      }
  }
   
  @FunctionTemplate(name = "extractQuarter", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class QuarterFromIntervalDay implements SimpleFunction {

    @Param IntervalDayHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
      out.value = 1;
      }
  }
   
   
 
   
  @FunctionTemplate(name = "extractSecond", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class SecondFromIntervalYear implements SimpleFunction {

    @Param IntervalYearHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
 
      out.value = 0;
    }
  }
   
  @FunctionTemplate(name = "extractMinute", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class MinuteFromIntervalYear implements SimpleFunction {

    @Param IntervalYearHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
 
      out.value = 0;
    }
  }
   
  @FunctionTemplate(name = "extractHour", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class HourFromIntervalYear implements SimpleFunction {

    @Param IntervalYearHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
 
      out.value = 0;
    }
  }
   
  @FunctionTemplate(name = "extractDay", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class DayFromIntervalYear implements SimpleFunction {

    @Param IntervalYearHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
 
      out.value = 0;
    }
  }
   
  @FunctionTemplate(name = "extractMonth", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class MonthFromIntervalYear implements SimpleFunction {

    @Param IntervalYearHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
 
      out.value = (in.value % org.apache.arrow.vector.util.DateUtility.yearsToMonths);
    }
  }
   
  @FunctionTemplate(name = "extractYear", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class YearFromIntervalYear implements SimpleFunction {

    @Param IntervalYearHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
 
      out.value = (in.value / org.apache.arrow.vector.util.DateUtility.yearsToMonths);
    }
  }
   
  @FunctionTemplate(name = "extractCentury", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class CenturyFromIntervalYear implements SimpleFunction {

    @Param IntervalYearHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
 
      out.value = (in.value / org.apache.arrow.vector.util.DateUtility.yearsToMonths)/100;
    }
  }
   
  @FunctionTemplate(name = "extractDecade", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class DecadeFromIntervalYear implements SimpleFunction {

    @Param IntervalYearHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
 
      out.value = (in.value / org.apache.arrow.vector.util.DateUtility.yearsToMonths)/10;
    }
  }
   
   
   
  @FunctionTemplate(name = "extractEpoch", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class EpochFromIntervalYear implements SimpleFunction {

    @Param IntervalYearHolder in;
    @Output BigIntHolder out;
    @Workspace org.joda.time.MutableDateTime dateTime;

    public void setup() {
      dateTime = new org.joda.time.MutableDateTime(
          org.joda.time.chrono.DayOfWeekFromSundayChronology.getISOInstanceInUTC());
    }

    public void eval() {
 
      dateTime.setMillis(0);
      dateTime.add(org.joda.time.DurationFieldType.years(), (in.value / org.apache.arrow.vector.util.DateUtility.yearsToMonths));
      dateTime.add(org.joda.time.DurationFieldType.months(), (in.value % org.apache.arrow.vector.util.DateUtility.yearsToMonths));
      out.value = dateTime.getMillis()/1000;
    }
  }
   
  @FunctionTemplate(name = "extractMillennium", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class MillenniumFromIntervalYear implements SimpleFunction {

    @Param IntervalYearHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
 
      out.value = (in.value / org.apache.arrow.vector.util.DateUtility.yearsToMonths)/1000;
    }
  }
   
  @FunctionTemplate(name = "extractQuarter", scope = FunctionTemplate.FunctionScope.SIMPLE,
      nulls = FunctionTemplate.NullHandling.NULL_IF_NULL)
  public static class QuarterFromIntervalYear implements SimpleFunction {

    @Param IntervalYearHolder in;
    @Output BigIntHolder out;

    public void setup() {
    }

    public void eval() {
 
      out.value = ((in.value % org.apache.arrow.vector.util.DateUtility.yearsToMonths)-1)/3 + 1;
    }
  }
   
   
 
}
