

package com.dremio.exec.expr.fn.impl;

import javax.inject.Inject;

import com.dremio.exec.expr.SimpleFunction;
import com.dremio.exec.expr.annotations.FunctionTemplate;
import com.dremio.exec.expr.annotations.FunctionTemplate.NullHandling;
import com.dremio.exec.expr.annotations.Output;
import com.dremio.exec.expr.annotations.Workspace;
import com.dremio.exec.expr.annotations.Param;
import com.dremio.exec.expr.fn.FunctionErrorContext;

import org.apache.arrow.vector.holders.*;

/**
 * generated from DateIntervalFunctionTemplates/IsDateFunctions.java DateMilli
 * implementations of IS_DATEMILLI(text, format) that checks if 'text' is a proper DATEMILLI if
 * formatted using the passed 'format' string.<br>
 * 'text' can be null, and the function returns FALSE in that case
 */
public class GIsDateMilliFunctions {
    @FunctionTemplate(name = "is_DATE" , scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.INTERNAL)
    public static class GIsDateMilli_Null implements SimpleFunction {

        @Param  
        NullableVarCharHolder left;
        
        @Param(constant=true)
        NullableVarCharHolder right;
        
        @Workspace 
        org.joda.time.format.DateTimeFormatter format;
        
        @Output 
        NullableBitHolder out;

        @Inject
        FunctionErrorContext errCtx;

        public void setup() {
            // Get the desired output format
            byte[] buf = new byte[right.end - right.start];
            right.buffer.getBytes(right.start, buf, 0, right.end - right.start);
            String formatString = new String(buf, java.nio.charset.StandardCharsets.UTF_8);
            format = com.dremio.exec.expr.fn.impl.DateFunctionsUtils.getSQLFormatterForFormatString(formatString, errCtx);
        }

        public void eval() {
            out.isSet = 1;
            if (left.isSet == 1) {
                // Get the input
                byte[]buf1=new byte[left.end-left.start];
                left.buffer.getBytes(left.start,buf1,0,left.end-left.start);
                String input=new String(buf1, java.nio.charset.StandardCharsets.UTF_8);

                try{
                    com.dremio.exec.expr.fn.impl.DateFunctionsUtils.formatDateMilli(input, format, errCtx);
                    out.value=1;
                } catch(Exception ex) {
                    // do not throw on purpose as we just want to set out.value
                }
            }
        }
    }

}

