

package com.dremio.exec.expr.fn.impl;

import javax.inject.Inject;

import com.dremio.exec.expr.SimpleFunction;
import com.dremio.exec.expr.annotations.FunctionTemplate;
import com.dremio.exec.expr.annotations.FunctionTemplate.NullHandling;
import com.dremio.exec.expr.annotations.Output;
import com.dremio.exec.expr.annotations.Workspace;
import com.dremio.exec.expr.annotations.Param;
import com.dremio.exec.expr.fn.FunctionErrorContext;

import org.apache.arrow.vector.holders.*;

/**
 * generated from DateIntervalFunctionTemplates/ToDateTypeFunctions.java TimeMilli
 */
public class GToTimeMilli {
  @FunctionTemplate(name = "to_TIME" , scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GVarCharToTimeMilli implements SimpleFunction{
    @Param VarCharHolder left;
    @Param VarCharHolder right;
    @Workspace org.joda.time.format.DateTimeFormatter format;
    @Workspace boolean replaceWithOffset;
    @Workspace int prevInputLength;
    @Workspace String prevTimezoneAbbr;
    @Output TimeMilliHolder out;
    @Inject FunctionErrorContext errCtx;

  public void setup(){
    // Get the desired output format
    byte[]buf=new byte[right.end-right.start];
    right.buffer.getBytes(right.start,buf,0,right.end-right.start);
    String formatString=new String(buf,java.nio.charset.StandardCharsets.UTF_8);
    if (formatString.toUpperCase().endsWith("TZD")) {
      formatString = formatString.substring(0, formatString.length() - 3) + "TZO";
      replaceWithOffset = true;
      prevInputLength = 0;
      prevTimezoneAbbr = "";
    }
    format=com.dremio.exec.expr.fn.impl.DateFunctionsUtils.getSQLFormatterForFormatString(formatString, errCtx);
  }

  public void eval(){
    // Get the input
    byte[]buf1=new byte[left.end-left.start];
    left.buffer.getBytes(left.start,buf1,0,left.end-left.start);
    String input=new String(buf1,java.nio.charset.StandardCharsets.UTF_8);
    if(replaceWithOffset) {
      String tzAbbr = "";
      if(prevInputLength == input.length() && input.endsWith(prevTimezoneAbbr)) {
        tzAbbr = prevTimezoneAbbr;
      } else {
        for(int i = input.length() - 1; i >= 0; i--) {
          char c = input.charAt(i);
          if( (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') ) {
            tzAbbr =  String.valueOf(c) + tzAbbr;
          } else {
            break;
          }
        }
        prevInputLength = input.length();
        prevTimezoneAbbr = tzAbbr;
      }

      java.util.Optional<String> offset =  com.dremio.exec.store.sys.TimezoneAbbreviations.getOffset(tzAbbr);
      if(!offset.isPresent()) {
          throw errCtx.error()
          .message("Input text cannot be formatted to date")
          .addContext("Invalid timezone abbreviation", tzAbbr)
          .addContext("Input text", input)
          .build();
      }

      input = input.substring(0, input.length() - tzAbbr.length()) + offset.get();
    }

    out.value=com.dremio.exec.expr.fn.impl.DateFunctionsUtils.formatTimeMilli(input, format, errCtx);
  }
  }

  @FunctionTemplate(name = "to_TIME", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GIntToTimeMilli implements SimpleFunction {
    @Param  IntHolder left;
    @Workspace org.joda.time.MutableDateTime mutableDateTime;
    @Output TimeMilliHolder out;
    @Inject FunctionErrorContext errCtx;

    public void setup() {
      mutableDateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
      long inputMillis = 0;
      inputMillis = (long) (left.value * 1000l);
      try {
        mutableDateTime.setMillis(inputMillis);
      } catch (IllegalArgumentException e) {
        errCtx.error()
          .message("argument '%s' is not a valid milliseconds value", inputMillis)
          .build();
      }
      out.value = mutableDateTime.getMillisOfDay();
    }
  }
  @FunctionTemplate(name = "to_TIME", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GBigIntToTimeMilli implements SimpleFunction {
    @Param  BigIntHolder left;
    @Workspace org.joda.time.MutableDateTime mutableDateTime;
    @Output TimeMilliHolder out;
    @Inject FunctionErrorContext errCtx;

    public void setup() {
      mutableDateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
      long inputMillis = 0;
      inputMillis = (long) (left.value * 1000l);
      try {
        mutableDateTime.setMillis(inputMillis);
      } catch (IllegalArgumentException e) {
        errCtx.error()
          .message("argument '%s' is not a valid milliseconds value", inputMillis)
          .build();
      }
      out.value = mutableDateTime.getMillisOfDay();
    }
  }
  @FunctionTemplate(name = "to_TIME", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GFloat4ToTimeMilli implements SimpleFunction {
    @Param  Float4Holder left;
    @Workspace org.joda.time.MutableDateTime mutableDateTime;
    @Output TimeMilliHolder out;
    @Inject FunctionErrorContext errCtx;

    public void setup() {
      mutableDateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
      long inputMillis = 0;
      inputMillis = (long) (left.value * 1000l);
      try {
        mutableDateTime.setMillis(inputMillis);
      } catch (IllegalArgumentException e) {
        errCtx.error()
          .message("argument '%s' is not a valid milliseconds value", inputMillis)
          .build();
      }
      out.value = mutableDateTime.getMillisOfDay();
    }
  }
  @FunctionTemplate(name = "to_TIME", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class GFloat8ToTimeMilli implements SimpleFunction {
    @Param  Float8Holder left;
    @Workspace org.joda.time.MutableDateTime mutableDateTime;
    @Output TimeMilliHolder out;
    @Inject FunctionErrorContext errCtx;

    public void setup() {
      mutableDateTime = new org.joda.time.MutableDateTime(org.joda.time.DateTimeZone.UTC);
    }

    public void eval() {
      long inputMillis = 0;
      inputMillis = (long) (left.value * 1000l);
      try {
        mutableDateTime.setMillis(inputMillis);
      } catch (IllegalArgumentException e) {
        errCtx.error()
          .message("argument '%s' is not a valid milliseconds value", inputMillis)
          .build();
      }
      out.value = mutableDateTime.getMillisOfDay();
    }
  }

  @FunctionTemplate(name = "to_TIME" , scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.INTERNAL)
  public static class GVarCharToTimeMilliWithReplaceOptions implements SimpleFunction{
    @Param VarCharHolder left;
    @Param(constant=true) VarCharHolder patternHolder;
    @Param(constant=true) private NullableIntHolder replaceWithNullHolder;
    @Workspace org.joda.time.format.DateTimeFormatter format;
    @Workspace boolean replaceWithOffset;
    @Workspace int prevInputLength;
    @Workspace String prevTimezoneAbbr;
    @Output NullableTimeMilliHolder out;
    @Inject FunctionErrorContext errCtx;

    public void setup() {
      // Get the desired output format
      byte[] buf = new byte[patternHolder.end-patternHolder.start];
      patternHolder.buffer.getBytes(patternHolder.start, buf, 0, patternHolder.end-patternHolder.start);

      // if the format string is invalid we want to fail, so don't handle the exception
      String formatString = new String(buf, java.nio.charset.StandardCharsets.UTF_8);
      if (formatString.toUpperCase().endsWith("TZD")) {
        formatString = formatString.substring(0, formatString.length() - 3) + "TZO";
        replaceWithOffset = true;
        prevInputLength = 0;
        prevTimezoneAbbr = "";
      }
      format=com.dremio.exec.expr.fn.impl.DateFunctionsUtils.getSQLFormatterForFormatString(formatString, errCtx);
  }

    public void eval() {
      if (left.isSet == 0) {
        out.isSet = 0;
        return;
      }

      byte[] buf = new byte[left.end - left.start];
      left.buffer.getBytes(left.start, buf, 0, left.end - left.start);
      String input = new String(buf, java.nio.charset.StandardCharsets.UTF_8);

      if(replaceWithOffset) {
        String tzAbbr = "";
        if(prevInputLength == input.length() && input.endsWith(prevTimezoneAbbr)) {
          tzAbbr = prevTimezoneAbbr;
        } else {
          for(int i = input.length() - 1; i >= 0; i--) {
            char c = input.charAt(i);
            if( (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') ) {
              tzAbbr =  String.valueOf(c) + tzAbbr;
            } else {
              break;
            }
          }
          prevInputLength = input.length();
          prevTimezoneAbbr = tzAbbr;
        }

        java.util.Optional<String> offset =  com.dremio.exec.store.sys.TimezoneAbbreviations.getOffset(tzAbbr);
        if(!offset.isPresent()) {
          throw errCtx.error()
            .message("Input text cannot be formatted to date")
            .addContext("Invalid timezone abbreviation", tzAbbr)
            .addContext("Input text", input)
            .build();
        }

        input = input.substring(0, input.length() - tzAbbr.length()) + offset.get();
      }

      try {
        out.value = com.dremio.exec.expr.fn.impl.DateFunctionsUtils.formatTimeMilli(input, format);
        out.isSet = 1;
      } catch (IllegalArgumentException e) {
        if (replaceWithNullHolder.value == 1) {
          out.isSet = 0;
        } else {
          // if replaceWithNullHolder is not set, we throw the exception to be consistent with to_TIME
          // calls without the replaceWithNullHolder argument
          throw errCtx.error()
            .message("Input text cannot be formatted to " + "TIME".toLowerCase())
            .addContext("Details", e.getMessage())
            .addContext("Input text", input)
            .build();
        }
      }
    }
  }
}

