

package com.dremio.exec.expr.fn.impl;

import io.netty.buffer.ByteBuf;

import org.apache.arrow.vector.holders.*;
import com.dremio.exec.expr.SimpleFunction;
import com.dremio.exec.expr.annotations.FunctionTemplate;
import com.dremio.exec.expr.annotations.FunctionTemplate.NullHandling;
import com.dremio.exec.expr.annotations.Output;
import com.dremio.exec.expr.annotations.Param;
import com.dremio.exec.expr.annotations.Workspace;
import org.joda.time.DateTimeZone;
import org.joda.time.DateMidnight;
import org.apache.arrow.vector.util.DateUtility;

/**
 * generated from DateIntervalFunctionTemplates/IntervalNumericArithmetic.java
 */
public class IntervalDayFloat4Functions {


    @SuppressWarnings("unused")
    @FunctionTemplate(name = "multiply", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls=NullHandling.NULL_IF_NULL)
    public static class IntervalDayFloat4MultiplyFunction implements SimpleFunction {
    @Param IntervalDayHolder left;
    @Param Float4Holder right;
    @Output IntervalDayHolder out;

        public void setup() {
        }

        public void eval() {
        long totalMillis = left.days * org.apache.arrow.vector.util.DateUtility.daysToStandardMillis + left.milliseconds;

        totalMillis = (long) (totalMillis * (double) right.value);

        int days = (int) (totalMillis / org.apache.arrow.vector.util.DateUtility.daysToStandardMillis);
        int millis = (int) (totalMillis - days * org.apache.arrow.vector.util.DateUtility.daysToStandardMillis);
        out.days = days;
        out.milliseconds = millis;
        }
    }

    @SuppressWarnings("unused")
    @FunctionTemplate(name = "multiply", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls=NullHandling.NULL_IF_NULL)
    public static class Float4IntervalDayMultiplyFunction implements SimpleFunction {
    @Param Float4Holder right;
    @Param IntervalDayHolder left;
    @Output IntervalDayHolder out;

        public void setup() {
        }

        public void eval() {
        long totalMillis = left.days * org.apache.arrow.vector.util.DateUtility.daysToStandardMillis + left.milliseconds;

        totalMillis = (long) (totalMillis * (double) right.value);

        int days = (int) (totalMillis / org.apache.arrow.vector.util.DateUtility.daysToStandardMillis);
        int millis = (int) (totalMillis - days * org.apache.arrow.vector.util.DateUtility.daysToStandardMillis);
        out.days = days;
        out.milliseconds = millis;
        }
    }

    @SuppressWarnings("unused")
    @FunctionTemplate(names = {"divide", "div"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls=NullHandling.NULL_IF_NULL)
    public static class IntervalDayFloat4DivideFunction implements SimpleFunction {
    @Param IntervalDayHolder left;
    @Param Float4Holder right;
    @Output IntervalDayHolder out;

        public void setup() {
        }

        public void eval() {
        long totalMillis = left.days * org.apache.arrow.vector.util.DateUtility.daysToStandardMillis + left.milliseconds;

        totalMillis = (long) (totalMillis / (double) right.value);

        int days = (int) (totalMillis / org.apache.arrow.vector.util.DateUtility.daysToStandardMillis);
        int millis = (int) (totalMillis - days * org.apache.arrow.vector.util.DateUtility.daysToStandardMillis);
        out.days = days;
        out.milliseconds = millis;
        }
    }
}

