

package com.dremio.exec.expr.fn.impl;

import io.netty.buffer.ByteBuf;

import com.dremio.exec.expr.SimpleFunction;
import com.dremio.exec.expr.annotations.FunctionTemplate;
import com.dremio.exec.expr.annotations.FunctionTemplate.NullHandling;
import com.dremio.exec.expr.annotations.Output;
import com.dremio.exec.expr.annotations.Param;
import com.dremio.exec.expr.annotations.Workspace;
import com.dremio.exec.expr.fn.FunctionErrorContext;
import org.apache.arrow.vector.holders.*;
import org.joda.time.MutableDateTime;
import org.joda.time.DateTimeZone;
import org.joda.time.DateMidnight;
import org.apache.arrow.vector.util.DateUtility;
import javax.inject.Inject;

/**
 * generated from DateIntervalFunctionTemplates/DateIntervalArithmeticFunctions.java TimeMilli IntervalDay
 */
public class TimeMilliIntervalDayFunctions {

    @SuppressWarnings("unused")
    @FunctionTemplate(names = {"date_add", "add"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls=NullHandling.NULL_IF_NULL)
    public static class TimeMilliIntervalDayAddFunction implements SimpleFunction {
    @Param TimeMilliHolder left;
    @Param IntervalDayHolder right;
    @Output TimeMilliHolder out;
    @Inject FunctionErrorContext errCtx;

        public void setup() {
        }

        @SuppressWarnings("IdentityBinaryExpression")
        public void eval() {
    out.value = left.value + right.milliseconds;
    // Wrap around 24 hour clock if we exceeded it while adding the time component
    out.value = out.value % org.apache.arrow.vector.util.DateUtility.daysToStandardMillis;
        }
    }

    @SuppressWarnings("unused")
    @FunctionTemplate(names = {"date_add", "add"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls=NullHandling.NULL_IF_NULL)
    public static class IntervalDayTimeMilliAddFunction implements SimpleFunction {
    @Param IntervalDayHolder right;
    @Param TimeMilliHolder left;
    @Output TimeMilliHolder out;
    @Inject FunctionErrorContext errCtx;

        public void setup() {
        }

        @SuppressWarnings("IdentityBinaryExpression")
        public void eval() {
    out.value = left.value + right.milliseconds;
    // Wrap around 24 hour clock if we exceeded it while adding the time component
    out.value = out.value % org.apache.arrow.vector.util.DateUtility.daysToStandardMillis;
        }
    }

    @SuppressWarnings("unused")
    @FunctionTemplate(names = {"date_sub", "subtract", "date_diff"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls=NullHandling.NULL_IF_NULL)
    public static class TimeMilliIntervalDaySubtractFunction implements SimpleFunction {
    @Param TimeMilliHolder left;
    @Param IntervalDayHolder right;
    @Output TimeMilliHolder out;
    @Inject FunctionErrorContext errCtx;

        public void setup() {
        }

        @SuppressWarnings("IdentityBinaryExpression")
        public void eval() {
    out.value = left.value - right.milliseconds;
    // Wrap around 24 hour clock if we exceeded it while adding the time component
    out.value = out.value % org.apache.arrow.vector.util.DateUtility.daysToStandardMillis;
        }
    }
}

