
package com.dremio.exec.expr.fn.impl;

import com.dremio.exec.expr.SimpleFunction;
import com.dremio.exec.expr.annotations.FunctionTemplate;
import com.dremio.exec.expr.annotations.FunctionTemplate.NullHandling;
import com.dremio.exec.expr.annotations.Output;
import com.dremio.exec.expr.annotations.Param;
import com.dremio.exec.expr.annotations.Workspace;
import org.apache.arrow.vector.holders.DateMilliHolder;
import org.apache.arrow.vector.holders.IntHolder;
import org.apache.arrow.vector.holders.BigIntHolder;
import org.apache.arrow.vector.holders.TimeStampMilliHolder;
import org.apache.arrow.vector.holders.VarCharHolder;
import org.joda.time.MutableDateTime;

/**
* generated from DateIntervalFunctionTemplates/TimestampArithmeticFunctions.java
*/
public class TimestampArithmeticFunctions {

 
 
 

  @FunctionTemplate(names = {"timestampaddMicroSecond", "add_MicroSeconds"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntMicroSecondToTimeStampMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MICROSECOND");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_MicroSeconds", "timestampaddMicroSecond"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntMicroSecondToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MICROSECOND");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddSecond", "add_Seconds"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntSecondToTimeStampMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("SECOND");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Seconds", "timestampaddSecond"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntSecondToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("SECOND");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddMinute", "add_Minutes"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntMinuteToTimeStampMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MINUTE");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Minutes", "timestampaddMinute"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntMinuteToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MINUTE");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddHour", "add_Hours"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntHourToTimeStampMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("HOUR");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Hours", "timestampaddHour"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntHourToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("HOUR");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddDay", "add_Days"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntDayToTimeStampMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("DAY");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Days", "timestampaddDay"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntDayToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("DAY");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddWeek", "add_Weeks"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntWeekToTimeStampMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("WEEK");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Weeks", "timestampaddWeek"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntWeekToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("WEEK");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddMonth", "add_Months"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntMonthToTimeStampMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MONTH");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Months", "timestampaddMonth"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntMonthToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MONTH");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddQuarter", "add_Quarters"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntQuarterToTimeStampMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("QUARTER");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Quarters", "timestampaddQuarter"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntQuarterToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("QUARTER");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddYear", "add_Years"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntYearToTimeStampMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("YEAR");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Years", "timestampaddYear"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntYearToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("YEAR");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 
 
 

  @FunctionTemplate(names = {"timestampaddMicroSecond", "add_MicroSeconds"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntMicroSecondToTimeStampMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MICROSECOND");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_MicroSeconds", "timestampaddMicroSecond"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntMicroSecondToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MICROSECOND");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddSecond", "add_Seconds"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntSecondToTimeStampMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("SECOND");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Seconds", "timestampaddSecond"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntSecondToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("SECOND");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddMinute", "add_Minutes"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntMinuteToTimeStampMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MINUTE");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Minutes", "timestampaddMinute"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntMinuteToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MINUTE");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddHour", "add_Hours"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntHourToTimeStampMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("HOUR");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Hours", "timestampaddHour"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntHourToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("HOUR");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddDay", "add_Days"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntDayToTimeStampMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("DAY");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Days", "timestampaddDay"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntDayToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("DAY");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddWeek", "add_Weeks"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntWeekToTimeStampMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("WEEK");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Weeks", "timestampaddWeek"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntWeekToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("WEEK");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddMonth", "add_Months"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntMonthToTimeStampMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MONTH");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Months", "timestampaddMonth"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntMonthToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MONTH");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddQuarter", "add_Quarters"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntQuarterToTimeStampMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("QUARTER");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Quarters", "timestampaddQuarter"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntQuarterToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("QUARTER");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddYear", "add_Years"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntYearToTimeStampMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param TimeStampMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("YEAR");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Years", "timestampaddYear"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntYearToTimeStampMilli implements SimpleFunction {
    @Param TimeStampMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output TimeStampMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("YEAR");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 
 
 
 
 
 
 
 
 
 
 
 
 

  @FunctionTemplate(names = {"timestampaddDay", "add_Days"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntDayToDateMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param DateMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("DAY");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Days", "timestampaddDay"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntDayToDateMilli implements SimpleFunction {
    @Param DateMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("DAY");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddWeek", "add_Weeks"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntWeekToDateMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param DateMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("WEEK");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Weeks", "timestampaddWeek"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntWeekToDateMilli implements SimpleFunction {
    @Param DateMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("WEEK");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddMonth", "add_Months"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntMonthToDateMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param DateMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MONTH");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Months", "timestampaddMonth"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntMonthToDateMilli implements SimpleFunction {
    @Param DateMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MONTH");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddQuarter", "add_Quarters"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntQuarterToDateMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param DateMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("QUARTER");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Quarters", "timestampaddQuarter"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntQuarterToDateMilli implements SimpleFunction {
    @Param DateMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("QUARTER");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddYear", "add_Years"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddIntYearToDateMilli implements SimpleFunction {
    @Param IntHolder count;
    @Param DateMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("YEAR");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Years", "timestampaddYear"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_IntYearToDateMilli implements SimpleFunction {
    @Param DateMilliHolder in;
    @Param IntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("YEAR");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 
 
 
 
 
 
 
 
 
 
 

  @FunctionTemplate(names = {"timestampaddDay", "add_Days"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntDayToDateMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param DateMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("DAY");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Days", "timestampaddDay"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntDayToDateMilli implements SimpleFunction {
    @Param DateMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("DAY");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddWeek", "add_Weeks"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntWeekToDateMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param DateMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("WEEK");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Weeks", "timestampaddWeek"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntWeekToDateMilli implements SimpleFunction {
    @Param DateMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("WEEK");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddMonth", "add_Months"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntMonthToDateMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param DateMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MONTH");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Months", "timestampaddMonth"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntMonthToDateMilli implements SimpleFunction {
    @Param DateMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MONTH");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddQuarter", "add_Quarters"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntQuarterToDateMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param DateMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("QUARTER");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Quarters", "timestampaddQuarter"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntQuarterToDateMilli implements SimpleFunction {
    @Param DateMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("QUARTER");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 

  @FunctionTemplate(names = {"timestampaddYear", "add_Years"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampAddBigIntYearToDateMilli implements SimpleFunction {
    @Param BigIntHolder count;
    @Param DateMilliHolder in;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("YEAR");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

  @FunctionTemplate(names = {"add_Years", "timestampaddYear"}, scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class Add_BigIntYearToDateMilli implements SimpleFunction {
    @Param DateMilliHolder in;
    @Param BigIntHolder count;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output DateMilliHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("YEAR");
    }

    public void eval() {
      temp.setMillis(in.value);
      tsi.addCount(temp, count.value);
      out.value = temp.getMillis();
    }
  }

 
 
 
 

 

  @FunctionTemplate(name = "timestampdiffMicroSecond", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampDiffMicroSecond implements SimpleFunction {
    @Param TimeStampMilliHolder left;
    @Param TimeStampMilliHolder right;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output IntHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MICROSECOND");
    }

    public void eval() {
      out.value = (int) ((left.value > right.value) ?
                  -tsi.getDiff(new org.joda.time.Interval(right.value, left.value, org.joda.time.DateTimeZone.UTC)) :
                  tsi.getDiff(new org.joda.time.Interval(left.value, right.value, org.joda.time.DateTimeZone.UTC)));
    }
  }

 

  @FunctionTemplate(name = "timestampdiffSecond", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampDiffSecond implements SimpleFunction {
    @Param TimeStampMilliHolder left;
    @Param TimeStampMilliHolder right;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output IntHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("SECOND");
    }

    public void eval() {
      out.value = (int) ((left.value > right.value) ?
                  -tsi.getDiff(new org.joda.time.Interval(right.value, left.value, org.joda.time.DateTimeZone.UTC)) :
                  tsi.getDiff(new org.joda.time.Interval(left.value, right.value, org.joda.time.DateTimeZone.UTC)));
    }
  }

 

  @FunctionTemplate(name = "timestampdiffMinute", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampDiffMinute implements SimpleFunction {
    @Param TimeStampMilliHolder left;
    @Param TimeStampMilliHolder right;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output IntHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MINUTE");
    }

    public void eval() {
      out.value = (int) ((left.value > right.value) ?
                  -tsi.getDiff(new org.joda.time.Interval(right.value, left.value, org.joda.time.DateTimeZone.UTC)) :
                  tsi.getDiff(new org.joda.time.Interval(left.value, right.value, org.joda.time.DateTimeZone.UTC)));
    }
  }

 

  @FunctionTemplate(name = "timestampdiffHour", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampDiffHour implements SimpleFunction {
    @Param TimeStampMilliHolder left;
    @Param TimeStampMilliHolder right;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output IntHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("HOUR");
    }

    public void eval() {
      out.value = (int) ((left.value > right.value) ?
                  -tsi.getDiff(new org.joda.time.Interval(right.value, left.value, org.joda.time.DateTimeZone.UTC)) :
                  tsi.getDiff(new org.joda.time.Interval(left.value, right.value, org.joda.time.DateTimeZone.UTC)));
    }
  }

 

  @FunctionTemplate(name = "timestampdiffDay", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampDiffDay implements SimpleFunction {
    @Param TimeStampMilliHolder left;
    @Param TimeStampMilliHolder right;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output IntHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("DAY");
    }

    public void eval() {
      out.value = (int) ((left.value > right.value) ?
                  -tsi.getDiff(new org.joda.time.Interval(right.value, left.value, org.joda.time.DateTimeZone.UTC)) :
                  tsi.getDiff(new org.joda.time.Interval(left.value, right.value, org.joda.time.DateTimeZone.UTC)));
    }
  }

 

  @FunctionTemplate(name = "timestampdiffWeek", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampDiffWeek implements SimpleFunction {
    @Param TimeStampMilliHolder left;
    @Param TimeStampMilliHolder right;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output IntHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("WEEK");
    }

    public void eval() {
      out.value = (int) ((left.value > right.value) ?
                  -tsi.getDiff(new org.joda.time.Interval(right.value, left.value, org.joda.time.DateTimeZone.UTC)) :
                  tsi.getDiff(new org.joda.time.Interval(left.value, right.value, org.joda.time.DateTimeZone.UTC)));
    }
  }

 

  @FunctionTemplate(name = "timestampdiffMonth", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampDiffMonth implements SimpleFunction {
    @Param TimeStampMilliHolder left;
    @Param TimeStampMilliHolder right;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output IntHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("MONTH");
    }

    public void eval() {
      out.value = (int) ((left.value > right.value) ?
                  -tsi.getDiff(new org.joda.time.Interval(right.value, left.value, org.joda.time.DateTimeZone.UTC)) :
                  tsi.getDiff(new org.joda.time.Interval(left.value, right.value, org.joda.time.DateTimeZone.UTC)));
    }
  }

 

  @FunctionTemplate(name = "timestampdiffQuarter", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampDiffQuarter implements SimpleFunction {
    @Param TimeStampMilliHolder left;
    @Param TimeStampMilliHolder right;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output IntHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("QUARTER");
    }

    public void eval() {
      out.value = (int) ((left.value > right.value) ?
                  -tsi.getDiff(new org.joda.time.Interval(right.value, left.value, org.joda.time.DateTimeZone.UTC)) :
                  tsi.getDiff(new org.joda.time.Interval(left.value, right.value, org.joda.time.DateTimeZone.UTC)));
    }
  }

 

  @FunctionTemplate(name = "timestampdiffYear", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls = NullHandling.NULL_IF_NULL)
  public static class TimestampDiffYear implements SimpleFunction {
    @Param TimeStampMilliHolder left;
    @Param TimeStampMilliHolder right;
    @Workspace MutableDateTime temp;
    @Workspace com.dremio.exec.util.TSI tsi;
    @Output IntHolder out;

    public void setup() {
      temp = new MutableDateTime(org.joda.time.DateTimeZone.UTC);
      tsi = com.dremio.exec.util.TSI.getByName("YEAR");
    }

    public void eval() {
      out.value = (int) ((left.value > right.value) ?
                  -tsi.getDiff(new org.joda.time.Interval(right.value, left.value, org.joda.time.DateTimeZone.UTC)) :
                  tsi.getDiff(new org.joda.time.Interval(left.value, right.value, org.joda.time.DateTimeZone.UTC)));
    }
  }

 
}
