

package com.dremio.exec.expr.fn.impl.gcast;

import io.netty.buffer.ByteBuf;

import com.dremio.exec.expr.SimpleFunction;
import com.dremio.exec.expr.annotations.FunctionTemplate;
import com.dremio.exec.expr.annotations.FunctionTemplate.NullHandling;
import com.dremio.exec.expr.annotations.Output;
import com.dremio.exec.expr.annotations.Param;
import com.dremio.exec.expr.annotations.Workspace;
import com.dremio.exec.expr.fn.FunctionErrorContext;
import org.apache.arrow.vector.holders.*;
import org.joda.time.DateTimeZone;
import org.joda.time.DateMidnight;
import org.apache.arrow.vector.util.DateUtility;
import javax.inject.Inject;
import org.apache.arrow.memory.ArrowBuf;

/**
 * generated from CastVarCharInterval.java VarChar IntervalDay VarCharInterval
 */
@SuppressWarnings("unused")
@FunctionTemplate(name = "castINTERVALDAY", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls=NullHandling.NULL_IF_NULL)
public class CastVarCharToIntervalDay implements SimpleFunction {

  @Param VarCharHolder in;
  @Output IntervalDayHolder out;
  @Inject FunctionErrorContext errCtx;
  @Workspace java.util.regex.Pattern AllNumbers;

  public void setup() {
      AllNumbers = java.util.regex.Pattern.compile("\\d+");
  }

  public void eval() {

      byte[] buf = new byte[in.end - in.start];
      in.buffer.getBytes(in.start, buf, 0, in.end - in.start);
      String input = new String(buf, com.google.common.base.Charsets.UTF_8);

      // Parse the ISO format
      org.joda.time.Period period;
      try {
        if (AllNumbers.matcher(input).matches()) {
          long temp = org.apache.calcite.runtime.SqlFunctions.toLong(input);
          long days = temp / org.apache.arrow.vector.util.DateUtility.daysToStandardMillis;
          long miliseconds = temp % org.apache.arrow.vector.util.DateUtility.daysToStandardMillis;
          out.days = Math.toIntExact(days);
          out.milliseconds = Math.toIntExact(miliseconds);
          return;
        }
        period = org.joda.time.Period.parse(input);
      } catch (RuntimeException e) {
        throw errCtx.error(e)
          .build();
      }

      final long millis = ((long)period.getHours() * org.apache.arrow.vector.util.DateUtility.hoursToMillis) +
                          ((long)period.getMinutes() * org.apache.arrow.vector.util.DateUtility.minutesToMillis) +
                          ((long)period.getSeconds() * org.apache.arrow.vector.util.DateUtility.secondsToMillis) +
                          ((long)period.getMillis());

      out.days         = period.getDays() +
                         org.joda.time.field.FieldUtils.safeToInt(millis / org.apache.arrow.vector.util.DateUtility.daysToStandardMillis);

      out.milliseconds = org.joda.time.field.FieldUtils.safeToInt(millis % org.apache.arrow.vector.util.DateUtility.daysToStandardMillis);

  }
}
 
  

